/**********************************************************************************************
  BurstPHCFile.c   - I/O routines for Photon count data

  3-17-09: In LoadPHCFile(), the incoming variable 'szFilename', which 
           is now the full-path filename, is no longer copied to the 
           PHOTON_STREAM struct's current base filename. 
 *********************************************************************************************/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "Burst.h"

/**************************************************************************************
  Create a PHC file header from info in PHOTON_STREAM struct
  *************************************************************************************/
void CreatePHCFileHeader(PHOTON_STREAM *ps, PHCFILE_HEADER *phc_hdr)
{
 
}

// Loads header and notes, returns TRUE if it is a PHC file
BOOL GetPHCFileHeader(char *szFilename,  PHCFILE_HEADER *fhd)
{
 char buf[10];
 FILE *fp = fopen(szFilename, "rb");

 fread(&buf, sizeof(char), 3, fp); 
 if(buf[0] == 'P' && buf[1] == 'H' && buf[2] == 'C') {
	 rewind(fp);
	 fread(fhd, sizeof(PHCFILE_HEADER), 1, fp);
     fclose(fp);
	 return(TRUE);
 }
 return(FALSE); 
}


// Get the data from a PHC file 
int LoadPHCFile(HWND hwnd, char *szFilename, PHOTON_STREAM *ps, double binwidth) 
{
  DWORD dwFileSizeHi, dwNumBytesRead, dwFileSize;
  int result; 
  LPBYTE lpByte;
  HANDLE hFile;
  PHCFILE_HEADER fhd;

  if(!GetPHCFileHeader(szFilename, &fhd)) return(NOT_A_PHCFILE);
  
  // load header info in PS struct:
  ps->BaseBinWidth = fhd.BinWidth;
  ps->BinWidth = fhd.BinWidth;
  ps->FileType = PHC_FILE; 
  ps->Hardware = fhd.Hardware;
  ps->PHCFiletype = fhd.DataType;
  //strcpy(ps->szFilename, szFilename);   // 3-17-09

  // open file for data read
  hFile = CreateFile(szFilename, GENERIC_READ, 0,NULL,OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if (hFile == INVALID_HANDLE_VALUE) return(FILE_OPEN_ERROR);
  dwFileSize = GetFileSize(hFile, &dwFileSizeHi); 

  // determine data storage scheme:
  // PHC_OLD   --- unsubtracted WORD data - old format - will probably never see again so neglect for now
  // PHC_BYTE  -- uncompressed BYTE data 
  // PHC_WORD  -- uncompressed WORD data - usually the result for rebinning a BYTE file
  // PHC_BYTE_COMP_4x4  or PHC_BYTE_COMP_3x5  or  PHC_BYTE_COMP_2x6  -- BYTE compressed data
  // where the byte has 4, 3 ,2 bits devoted to the repeat value and the rest of thebyte is the count value
  // PHC_WORD_COMPRESSED  --- WORD compressed -- work out details later since it may not be needed
  // PHC_COMPRESSED_OTHER -- Alternative compression - for future use
  lpByte = (BYTE *) GetVirtualMem(dwFileSize);  // get virtual mem
  if(lpByte == NULL) return(MEM_ERROR);
  ReadFile(hFile, lpByte, dwFileSize, &dwNumBytesRead, NULL);
  if(dwNumBytesRead == 0) return(ZERO_BYTES_READ); 
  lpByte = lpByte + PHC_HEADER_SIZE ;    // move ptr past header

  // move data from the file buffer to the ps.counts array
  if((fhd.DataType > PHC_WORD) && (fhd.DataType < PHC_WORD_COMPRESSED)) {
      result = LoadCompressedBytePHC(ps, &fhd, lpByte);
  }
  else {
	  switch(fhd.DataType) {
	  case PHC_BYTE: LoadUnCompressedBytePHC(ps, &fhd, lpByte); break;  	 // direct load BYTE data
	  case PHC_WORD: LoadUnCompressedWordPHC(ps, &fhd, lpByte); break;	 // direct load WORD data
      case PHC_WORD_COMPRESSED:   // fall through 
	  case PHC_COMPRESSED_OTHER: 
		  MessageBox(hwnd, "This type of PHC file is not yet supported.", "PHC file error", MB_OK);
		  return(NOT_A_PHCFILE);
	  }
  }
	  
  CloseHandle(hFile);
  return(NO_ERROR);
}


int LoadCompressedBytePHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte)
{
 return(1);
}

int LoadUnCompressedBytePHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte)
{
 return(1);
}


int LoadUnCompressedWordPHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte)
{
 return(1);
}


int SavePHCFile(HWND hwnd, PHOTON_STREAM *ps, BOOL bCompress)
{
  char szFilter[] = "Photon Count File (*.phc)\0*.phc\0";
  char szExt[] = ".phc";
  char szFilename[256];
  PHCFILE_HEADER fhd;
  FILE *fp;
  LPBYTE lpByte;
  BYTE DataType;
  DWORD dwSize;

 // get filename
 strcpy(szFilename, ps->szFilename);
 remove_fext(szFilename);
 strcat(szFilename, ".phc");
 strcpy(szFilename, GetSaveFile(hwnd, szFilename, szFilter, szExt));
 if(szFilename[0] == '\0') return(USER_CANCEL);
 
 // load header info 
 fhd.szPHC[0] = 'P'; 
 fhd.szPHC[1] = 'H';
 fhd.szPHC[2] = 'C'; 
 fhd.BinWidth = (WORD) ps->BinWidth;
 fhd.Hardware = ps->Hardware;
 fhd.max = (WORD) ps->max;
 fhd.nbins = ps->nbins;
 fhd.Power = 1;
 fhd.PowerCode = 1; 
 fhd.reserved[0] = 0;
 fhd.reserved[1] = 0;
 //fhd.szNotes

 // open file for writing 
 fp = fopen(szFilename, "wb");
 if(fp == NULL) return(FILE_OPEN_ERROR);  

 // compress data if requested
 if (bCompress) {
    if(ps->max < 64) {
       lpByte = (BYTE *) GetVirtualMem(2 * ps->nbins);    // get mem for compression algorithm
	   if(!lpByte) return(MEM_ERROR);
       CompressPSDataByte(ps, &DataType, &dwSize, lpByte);  // compress in byte mode
       fhd.DataType = DataType;
       fhd.nvals = dwSize; 
       fwrite(&fhd, sizeof(PHCFILE_HEADER), 1, fp);    	  // write headerinfo
	   fwrite(lpByte, sizeof(BYTE), (size_t) dwSize, fp);
       fclose(fp);
	   VirtualFree(lpByte, 0, MEM_RELEASE);
       lpByte = NULL; 
	   return(NO_ERRORS);
	}
    else {
	  return(NO_ERRORS);
	}
 }
 	
    
  return(FALSE);
 }



int CompressPSDataByte(PHOTON_STREAM *ps, BYTE *DataType, DWORD *dwSize, LPBYTE bdata)
{
 DWORD i, j, k, nbins, bufsize;
 WORD max;
 BYTE nbits, rmax, value, cnt;
 
 nbins = ps->nbins; 
 bufsize = 2 * nbins;
 max = ps->max; 

 if(!(max & 0x11110000)) {
	 *DataType = PHC_BYTE_COMP_4x4;
	 rmax = 16;       // maximum repeat value
	 nbits = 4;       // number of bits that make up the repeat value
 }
 else if(!(max & 0x11100000)) {
	     *DataType = PHC_BYTE_COMP_3x5;
	     rmax = 8;
		 nbits = 3;
 }
 else if(!(max & 0x11000000)) {
	     *DataType = PHC_BYTE_COMP_2x6;
	     rmax = 4;
		 nbits = 2;
 }
 	 
	 
 i = k = 0;
 while(i < nbins) {
		 j = 1;
		 cnt = 1;
		 while((ps->counts[i] == ps->counts[i+j]) && (cnt < rmax)) {
			 cnt++;
			 j++;
		 }
		 value = (BYTE) ps->counts[i];
		 i += j;
		 bdata[k] = MAKEBYTE(cnt, nbits, value);
		 k++;
	 }
	 *dwSize = k;
 return(0);
}


int CompressPSDataWord(PHOTON_STREAM *ps, BYTE *DataType, DWORD *dwSize, LPWORD wdata)
{
 DWORD i, j, k, nbins, bufsize;
 WORD max;
 BYTE nbits, rmax, value, cnt;
 
 nbins = ps->nbins; 
 bufsize = 2 * nbins;
 max = ps->max; 

 *DataType = PHC_WORD_COMPRESSED;
 rmax = 255;
 nbits = 8;
	 
 i = k = 0;
 while(i < nbins) {
		 j = 1;
		 cnt = 1;
		 while((ps->counts[i] == ps->counts[i+j]) && (cnt < rmax)) {
			 cnt++;
			 j++;
		 }
		 value = (BYTE) ps->counts[i];
		 i += j;
		// bdata[k] = MAKEWORD(cnt, nbits, value);
		 k++;
	 }
	 *dwSize = k;
 return(0);
}
