/**********************************************************************************************
   PHCFile.h   Header file that defines the PHC file format used to store photon count data.
   
***********************************************************************************************/
#pragma once

// Defines for DataType variable in PHC header (how the data is stored)
#define PHC_OLD                 0   // unsubtracted WORD data - old format - will probably never use
#define PHC_BYTE                1   // BYTE data with header (uncompressed)
#define PHC_WORD                2   // WORD data with header (uncompressed)
#define PHC_BYTE_COMP_4x4       3   // compression format - see additional header info to decode
#define PHC_BYTE_COMP_3x5       4   // compression format - see additional header info to decode
#define PHC_BYTE_COMP_2x6       5   // compression format - see additional header info to decode
#define PHC_WORD_COMPRESSED     6   // compression format - see additional header info to decode
#define PHC_COMPRESSED_OTHER    7   // Alternative compression - for future use

// Hardware codes (how the data was originally obtained) - defines for Hardware variable in PHC header:
#define ADVANTECH1755           0   // Advantech PCI-1755
#define ADLINK7200              1   // Adlink PCI-7200
#define COR_COM                 2   // Correlator.Com card 
#define SPC_830                 3   // B&H SPC format from 830 card
#define HAMA_FORMAT             4   // Hamamatsu correlator card format

// Laser power codes
#define NOT_MEASURED            0
#define PERCENT_T               1
#define ACTUAL_POWER            2

// PHC file header sizes
#define PHC_HEADER_SIZE         1024   // # bytes in header 
#define PHC_HEADER_NOTES        1000   // # char's in notes section

// macros needed
#define MAKEBYTE(nrep, nbits, value)      ((nrep << nbits) | value)
#define GETREPEATVALUE(byte, nbits)        (byte >> nbits)
#define GETCOUNTVALUE(byte, nbits)         ((byte << nbits) >> nbits)


// Photon count (PHC) file header structure: version 1
typedef struct tagPHCFILE_HEADER {
   char   szPHC[3];       // "PHC" if a valid phc file                                             3 bytes            
   BYTE   Hardware;       // type of hardware used to acquire the data                             4 bytes
   BYTE   DataType;       // uncompressed or compressed, BYTE or WORD  - see defines above         5 bytes
   DWORD  nvals;          // number of values in compressed file (or = nbins if uncompressed)      9 bytes
   DWORD  nbins;          // number points (bins) in file                                         13 bytes  
   WORD   max;            // maximum data value                                                   15 bytes    
   WORD   BinWidth;       // binwidth in microseconds  (usually 1 s)                             17 bytes                     
   BYTE   PowerCode;      // type of power measurement  (see above defines)                       18 bytes
   float  Power;          // laser power                                                          22 bytes
   BYTE   reserved[2];    // reserved for future use                                              24 bytes
   char   szNotes[1000];  // user notes  - limit to 1000 char's                                 1024 bytes
}  PHCFILE_HEADER;                                       


